@@ -12,10 +12,10 @@ def get_sharpness(image):
     return sharpness
 
 
-def convert_cam_to_instant_ngp_json(
+def convert_cam_to_nerf_json(
     cam,
     json_path=None,
-    target_scale=None,
+    target_scale=None,  # 1~5m, recommend 2m
     aabb_scale=4,
     sharpness=True,
     basename=False,
@@ -38,6 +38,8 @@ def convert_cam_to_instant_ngp_json(
     nerf_json.update(distort)
     nerf_json["frames"] = []
     for key, d in cam.items():
+        if "T" not in d:
+            continue
         if "T_cam_in_world" in d:
             T_cam_in_world = d["T_cam_in_world"]
         else:
@@ -50,7 +52,12 @@ def convert_cam_to_instant_ngp_json(
             )
             T_cam_in_world = T_cam_in_world @ cv_to_nerf
             if rotate_z_as_up:
-                T_cam_in_world = utils.r_t_to_T([np.pi, 0, 0],) @ T_cam_in_world
+                T_cam_in_world = (
+                    utils.r_t_to_T(
+                        [np.pi, 0, 0],
+                    )
+                    @ T_cam_in_world
+                )
         dic = dict(
             file_path=boxx.basename(d["path"]) if basename else d["path"],
             transform_matrix=T_cam_in_world,
@@ -72,16 +79,19 @@ def convert_cam_to_instant_ngp_json(
 
 if __name__ == "__main__":
     import os
-    from calibrating import CharucoFeatureLib, Cam
+    from calibrating import CharucoBoard, Cam
     from boxx import *
 
     example_data_dir = os.path.abspath(
-        os.path.join(__file__, "../../../calibrating_example_data",)
+        os.path.join(
+            __file__,
+            "../../../calibrating_example_data",
+        )
     )
     recon_dir = os.path.join(example_data_dir, "reconstruction_with_marker_board")
     glob_path = recon_dir + "/*.jpg"
 
-    feature_lib = CharucoFeatureLib.build_with_calibration_img(
+    board = CharucoBoard.build_with_calibration_img(
         ppi=218.35,
         hw=(2480, 3508),
         n=12,
@@ -89,7 +99,10 @@ if __name__ == "__main__":
         using_marker_corner=True,
     )
 
-    cam = Cam(sorted(boxx.glob(glob_path)), feature_lib,)
+    cam = Cam(
+        sorted(boxx.glob(glob_path)),
+        board,
+    )
     jsp = recon_dir + "/transforms.json"
-    convert_cam_to_instant_ngp_json(cam, json_path=jsp, basename=True, target_scale=2.5)
+    convert_cam_to_nerf_json(cam, json_path=jsp, basename=True, target_scale=2.5)
     print("Save to:", jsp)
