@@ -31,12 +31,22 @@ class Stereo:
         └── D: (1, 5)float64
     """
 
-    def __init__(self, cam1=None, cam2=None, force_same_intrinsic=False):
+    def __init__(
+        self,
+        cam1=None,
+        cam2=None,
+        force_same_intrinsic=False,
+        xy_target=None,
+        K_target=1,
+    ):
+        self.xy_target = xy_target
+        self.K_target = K_target
         if cam1 is None:
             return
         self.cam1 = cam1
         self.cam2 = cam2
         self.force_same_intrinsic = force_same_intrinsic
+        # TODO better cx cy
         self.get_R_t_by_stereo_calibrate()
         self._get_undistort_rectify_map()
 
@@ -111,31 +121,87 @@ class Stereo:
         self.retval = retval
 
     def _get_undistort_rectify_map(self):
-        xy = self.cam1.xy
-        self.R1, self.R2, P1, P2, Q, validPixROI1, validPixROI2 = cv2.stereoRectify(
-            self.cam1.K, self.cam1.D, self.cam2.K, self.cam2.D, xy, self.R, self.t
-        )
+
+        if self.xy_target is None:
+            self.xy_target = self.cam1.xy
+        if isinstance(self.xy_target, (int, float)):
+            self.xy_target = [int(round(i * self.xy_target)) for i in self.cam1.xy]
+        xy = tuple(self.xy_target)
+        if isinstance(self.K_target, (int, float)):
+            K_target = self.K_target
+            self.K_target = self.cam1.K.copy()
+            self.K_target[:2, :2] *= K_target
+            self.K_target[:2, 2] += (np.array(xy) - self.cam1.xy) / 2
+
+        self.stereo_recitfy()
+
         self.undistort_rectify_map1 = cv2.initUndistortRectifyMap(
-            self.cam1.K, self.cam1.D, self.R1, self.cam1.K, xy, cv2.CV_32FC1
-        )
-        self.undistort_rectify_map2 = cv2.initUndistortRectifyMap(
-            self.cam2.K, self.cam2.D, self.R2, self.cam1.K, xy, cv2.CV_32FC1
+            self.cam1.K, self.cam1.D, self.R1, self.K_target, xy, cv2.CV_32FC1
         )
 
+        self.undistort_rectify_map2 = cv2.initUndistortRectifyMap(
+            self.cam2.K, self.cam2.D, self.R2, self.K_target, xy, cv2.CV_32FC1
+        )
+        assert (utils.R_t_to_T(self.R2.T) @ utils.R_t_to_T(self.R, self.t))[
+            0, 3
+        ] < 0, "The left and right cameras may be wrong. Please try to switch the camera order of Stereo(cam1, cam2)"
+
+    def stereo_recitfy_by_cv2(self):
+        """When the right eye camera is rotated 180° around the z axis, a BUG will appear"""
+        self.R1, self.R2, P1, P2, Q, validPixROI1, validPixROI2 = cv2.stereoRectify(
+            self.cam1.K,
+            self.cam1.D,
+            self.cam2.K,
+            self.cam2.D,
+            tuple(self.xy_target),
+            self.R,
+            self.t,
+        )
+
+    def stereo_recitfy(self):
+        axes_z = np.array([0, 0, 1.0])
+        axes_nx = np.array([-1.0, 0, 0])
+        t = self.t.squeeze()
+        plane_v = t
+        z_on_plane2 = utils.project_vec_on_plane(axes_z, plane_v)
+        z_on_plane1 = utils.project_vec_on_plane(self.R @ axes_z, plane_v)
+        z_on_plane = z_on_plane2 / np.linalg.norm(
+            z_on_plane2
+        ) + z_on_plane1 / np.linalg.norm(z_on_plane1)
+
+        R_align_x = utils.rotate_shortest_of_two_vecs(axes_nx, t)
+        R_align_z = utils.rotate_shortest_of_two_vecs(R_align_x @ axes_z, z_on_plane)
+
+        self.R2 = (R_align_z @ R_align_x).T
+        self.R1 = self.R2 @ self.R[:3, :3]
+
     def rectify(self, img1, img2):
-        rectified1 = cv2.remap(
+        rectify_img1 = cv2.remap(
             self._get_img(img1),
             self.undistort_rectify_map1[0],
             self.undistort_rectify_map1[1],
             cv2.INTER_LANCZOS4,
         )
-        rectified2 = cv2.remap(
+        rectify_img2 = cv2.remap(
             self._get_img(img2),
             self.undistort_rectify_map2[0],
             self.undistort_rectify_map2[1],
             cv2.INTER_LANCZOS4,
         )
-        return [rectified1, rectified2]
+
+        if getattr(self, "translation_rectify_img", None):
+            if self.min_disparity > 0:
+                rectify_img2[:, self.min_disparity :] = rectify_img2[
+                    :, : -self.min_disparity
+                ]
+                rectify_img2[:, : self.min_disparity] = 0
+            elif self.min_disparity < 0:
+                rectify_img2[:, : self.min_disparity] = rectify_img2[
+                    :, -self.min_disparity :
+                ]
+                rectify_img2[:, self.min_disparity :] = 0
+
+        return [rectify_img1, rectify_img2]
 
     DUMP_ATTRS = ["R", "t", "retval"]
 
@@ -222,15 +288,13 @@ class Stereo:
         K = self.cam1.K
         baseline = self.baseline
         zmin, zmax = min(depth_limits), max(depth_limits)
-        print(
-            "-" * 15, "Stereo.precision_analysis", "-" * 15,
-        )
+        print("-" * 15, "Stereo.precision_analysis", "-" * 15)
         print(self)
         print("\n")
         dispmin = baseline * K[0, 0] / zmax
         dispmax = baseline * K[0, 0] / zmin
         disps = np.linspace(dispmin, dispmax)
-        uncen_on_disp = -baseline * K[0, 0] / disps ** 2
+        uncen_on_disp = -baseline * K[0, 0] / disps**2
         zs = baseline * K[0, 0] / disps
         disp_on_z = baseline * K[0, 0] / zs
 
@@ -250,9 +314,7 @@ class Stereo:
         plt.grid()
         plt.show()
         # TODO: x,y 双目共同视野 - depth 关系
-        print(
-            "-" * 15, "End of Stereo.precision_analysis", "-" * 15,
-        )
+        print("-" * 15, "End of Stereo.precision_analysis", "-" * 15)
 
     def __str__(self):
         r = utils.T_to_r(self.R).squeeze()
@@ -296,22 +358,29 @@ class Stereo:
 
     @property
     def baseline(self):
-        return np.sum(self.t ** 2) ** 0.5
+        return np.sum(self.t**2) ** 0.5
 
     def depth_to_disparity(self, depth):
-        fx = self.cam1.K[0, 0]
+        fx = self.K_target[0, 0]
         disparity = 1.0 * self.baseline * fx / depth
         return disparity
 
     def disparity_to_depth(self, disparity):
-        fx = self.cam1.K[0, 0]
+        fx = self.K_target[0, 0]
         depth = 1.0 * self.baseline * fx / disparity
         depth[depth > self.get_max_depth()] = 0
         return depth
 
     def unrectify_depth(self, depth):
         maps = getattr(self, "_unrectify_depth_maps", None)
-        re = utils.rotate_depth_by_remap(self.cam1.K, self.R1.T, depth, maps=maps)
+        re = utils.rotate_depth_by_remap(
+            self.K_target,
+            self.R1.T,
+            depth,
+            maps=maps,
+            xy_target=self.cam1.xy,
+            K_target=self.cam1.K,
+        )
         if maps is None:
             self._unrectify_depth_maps = re["maps"]
         rotated_depth = re["depth"]
@@ -364,7 +433,7 @@ class Stereo:
         max_depth : float, optional
             The default is Stereo.MAX_DEPTH.
         translation_rectify_img : bool, the default is None.
-            When self.get_depth(), translation rectify_img2 by self.min_disparity, 
+            When self.get_depth(), translation rectify_img2 by self.min_disparity,
             self.min_disparity is accroding to self.max_depth.
             default is accroding to bool(max_depth)
         """
@@ -392,12 +461,6 @@ class Stereo:
             self, "stereo_matching"
         ), "Please stereo.set_stereo_matching(stereo_matching)"
         rectify_img1, rectify_img2 = self.rectify(img1, img2)
-        if getattr(self, "translation_rectify_img"):
-            rectify_img2[:, self.min_disparity :] = rectify_img2[
-                :, : -self.min_disparity
-            ]
-            rectify_img2[:, : self.min_disparity] = 0
-        # shows-(rectify_img1, rectify_img2)
         disparity = self.stereo_matching(rectify_img1, rectify_img2)
         if getattr(self, "translation_rectify_img"):
             disparity += self.min_disparity
@@ -414,11 +477,13 @@ class Stereo:
             unrectify_depth = self.unrectify_depth(rectify_depth)
             undistort_img1 = self.undistort_img(img1)
             result.update(
-                unrectify_depth=unrectify_depth, undistort_img1=undistort_img1,
+                unrectify_depth=unrectify_depth,
+                undistort_img1=undistort_img1,
             )
         if return_distort_depth:
             result.update(
-                distort_img1=img1, distort_depth=self.distort_depth(unrectify_depth),
+                distort_img1=img1,
+                distort_depth=self.distort_depth(unrectify_depth),
             )
         return result
 
@@ -486,23 +551,23 @@ class SemiGlobalBlockMatching(MetaStereoMatching):
         return disparity
 
 
-class MatchingByFeatureLib(MetaStereoMatching):
+class MatchingByBoard(MetaStereoMatching):
     """
-    Stereo mathch disp by feature_lib's image_points
+    Stereo mathch disp by board's image_points
     return a dense disp on calibration board
     """
 
-    def __init__(self, feature_lib, dense_predict=True):
-        self.feature_lib = feature_lib
+    def __init__(self, board, dense_predict=True):
+        self.board = board
         self.dense_predict = dense_predict
 
     def __call__(self, img1, img2):
-        self.feature_lib
+        self.board
         d1 = dict(img=img1)
-        self.feature_lib.find_image_points(d1)
+        self.board.find_image_points(d1)
         image_points1 = d1["image_points"]
         d2 = dict(img=img2)
-        self.feature_lib.find_image_points(d2)
+        self.board.find_image_points(d2)
         image_points2 = d2["image_points"]
 
         if isinstance(image_points1, dict):
