@@ -5,8 +5,9 @@ import cv2
 import boxx
 import numpy as np
 from glob import glob
+from boxx import imread, imsave, shows, show, showb, tree, loga, pi
 
-from boxx import imread, shows, show, showb
+inv = np.linalg.inv
 
 
 def R_t_to_T(R, t=None):
@@ -14,13 +15,13 @@ def R_t_to_T(R, t=None):
         t = np.zeros((3,))
     T = np.zeros((4, 4))
     T[:3, :3] = R
-    T[:3, -1] = t.squeeze()
+    T[:3, -1] = np.array(t).squeeze()
     T[3, 3] = 1
     return T
 
 
 def r_t_to_T(r, t=None):
-    r = np.array(r)
+    r = np.float32(r)
     assert r.size == 3
     return R_t_to_T(cv2.Rodrigues(r.squeeze())[0], t)
 
@@ -49,7 +50,7 @@ def R_to_deg(r_R_T):
 
 def T_to_deg_distance(T, compare_T=None):
     """
-    Translate the T matrix into items that humans can intuitively understand, 
+    Translate the T matrix into items that humans can intuitively understand,
     such as degrees of rotation and displacement distances
     Set compare_T to compare the difference between two Ts
 
@@ -76,6 +77,19 @@ def convert_points_for_cv2(dic_or_np):
     return point
 
 
+def project_vec_on_plane(v, plane_v):
+    return v - np.dot(v, plane_v) / (np.linalg.norm(plane_v) ** 2) * plane_v
+
+
+def rotate_shortest_of_two_vecs(v1, v2, return_rodrigues=False):
+    cross = np.cross(v1, v2)
+    rad = np.arccos((v1 * v2).sum() / np.linalg.norm(v1) / np.linalg.norm(v2))
+    r = rad * cross / np.linalg.norm(cross)
+    if return_rodrigues:
+        return r
+    return cv2.Rodrigues(r)[0]
+
+
 def apply_T_to_point_cloud(T, point_cloud):
     """
     point_cloud's shape is N * [x, y, z] or N * [x, y, z, ....]
@@ -97,10 +111,25 @@ def rotate_depth_by_point_cloud(K, R, depth, interpolation_rate=1):
     return dict(depth=new_depth, R=R)
 
 
-def rotate_depth_by_remap(K, R, depth, maps=None):
+def rotate_depth_by_remap(K, R, depth, maps=None, xy_target=None, K_target=None):
     y, x = depth.shape
+    if K_target is None:
+        K_target = K
+    if xy_target is None:
+        xy_target = x, y
+    else:
+        # With the optical center as the midpoint, expand image xy
+        K_target = K_target.copy()
+        # K_target[:2, 2] += (np.array(xy_target) - (x, y))/2
     if maps is None:
-        maps = cv2.initUndistortRectifyMap(K, None, R, K, (x, y), cv2.CV_32FC1,)
+        maps = cv2.initUndistortRectifyMap(
+            K,
+            None,
+            R,
+            K_target,
+            xy_target,
+            cv2.CV_32FC1,
+        )
     ys, xs = np.mgrid[:y, :x]
     ys, xs = ys.flatten(), xs.flatten()
     points = np.array([xs, ys, np.ones_like(xs)]) * depth.flatten()[None]
@@ -131,7 +160,10 @@ def depth_to_point_cloud(depth, K, interpolation_rate=1, return_xyzuv=False):
         depth = np.float32(depth / 1000.0)
     if interpolation_rate == 1:
         mask = depth != 0
-        vs, us = np.mgrid[:y, :x,][:, mask]
+        vs, us = np.mgrid[
+            :y,
+            :x,
+        ][:, mask]
 
         # points = (np.array([xs, ys, np.ones_like(xs)]) * depth[None])[:, depth != 0].T
         points = (np.array([us, vs, np.ones_like(us)]) * depth[mask]).T
@@ -140,7 +172,13 @@ def depth_to_point_cloud(depth, K, interpolation_rate=1, return_xyzuv=False):
         y_, x_ = int(round(y * interpolation_rate)), int(round(x * interpolation_rate))
         depth_ = cv2.resize(depth, (x_, y_), interpolation=cv2.INTER_NEAREST)
         mask = depth_ != 0
-        vs, us = np.mgrid[:y_, :x_,][:, mask] / interpolation_rate
+        vs, us = (
+            np.mgrid[
+                :y_,
+                :x_,
+            ][:, mask]
+            / interpolation_rate
+        )
         points = (np.array([us, vs, np.ones_like(us)]) * depth_[mask]).T
 
     point_cloud = (np.linalg.inv(K) @ points.T).T
@@ -155,7 +193,30 @@ def point_cloud_to_depth(points, K, xy):
 
 
 def point_cloud_to_arr2d(points, K, xy, values=None, bg_value=0):
-    xyzs = (K @ points.T).T
+    """
+    Converts 3D point cloud data into a 2D image array.
+
+    Parameters
+    ----------
+    points : ndarray
+        A Nx3 array of 3D points.
+    K : ndarray
+        A 3x3 matrix used to perform a projection transformation on the points.
+    xy : tuple of int
+        A tuple of the form (width, height) specifying the dimensions of the
+        desired 2D image array.
+    values : ndarray, optional
+        A 2D array of shape (N, C) representing data associated with each 3D point. Defaults to None.
+    bg_value : int, optional
+        The background value of the generated 2D image array.
+
+    Returns
+    -------
+    arr2d : ndarray
+        A 2D image array of shape (y, x, C) representing the converted 3D point cloud.
+    """
+    # equl to xyzs = (K @ points.T).T
+    xyzs = points @ K.T
     xyzs[:, :2] /= xyzs[:, 2:]
     sorted_idx = np.argsort(-xyzs[:, 2])
     xyzs_sorted = xyzs[sorted_idx]
@@ -169,6 +230,13 @@ def point_cloud_to_arr2d(points, K, xy, values=None, bg_value=0):
 
 
 def uvzs_to_arr2d(uvs, hw=None, bg_value=0, arr2d=None, values=None):
+    """
+    uvs: numpy array with shape (n, 2) or (n, 2 + values.shape[1]), where n is the number of (u, v) coordinates
+    hw: tuple of ints, representing the height and width of the resulting array. If not specified, it will default to the maximum (u, v) coordinates rounded up.
+    bg_value: int or float, representing the background value to fill the resulting array with.
+    arr2d: default arr2d. If not specified, a new array will be created with the specified height and width.
+    values: numpy array with shape (n, values.shape[1]), where values.shape[1] is the number of values to be associated with each (u, v) coordinate.
+    """
     if values is None:
         uvs, values = uvs[:, :2], uvs[:, 2:]
     if values.ndim == 1:
@@ -190,12 +258,6 @@ def uvzs_to_arr2d(uvs, hw=None, bg_value=0, arr2d=None, values=None):
     return arr2d
 
 
-# TODO rm
-def xyzs_to_arr2d(*args, **argkws):
-    print("\n\nWarning: xyzs_to_arr2d have rename to uvzs_to_arr2d")
-    return uvzs_to_arr2d(*args, **argkws)
-
-
 def arr2d_to_uvzs(arr2d, mask=None):
     y, x = arr2d.shape
     ys, xs = np.mgrid[:y, :x]
@@ -250,7 +312,10 @@ def interpolate_sparse2d(sparse2d, constrained_type=None, inter_type="lstsq"):
             smooth=0.5,
             episilon=5,
         )
-        output = spline(input_uvs[:, 0], input_uvs[:, 1],)
+        output = spline(
+            input_uvs[:, 0],
+            input_uvs[:, 1],
+        )
         output_uvzs = np.append(input_uvs, output[:, None], axis=-1)
     elif inter_type == "lstsq":
         A = uvzs.copy()
@@ -338,7 +403,7 @@ def vis_depth_l1(re, gt=0, max_l1=None, overexposed=True):
     Pretty vis of depth l1, which will ignore missing depth.
     Could distinguish missing depth(black) and l1==0(grey)
     For l1>0(far) red, l1<0(near) green, color of overexposed area will turns white
-    
+
     Parameters
     ----------
     re : depth
@@ -410,6 +475,8 @@ def vis_point_uvs(
         size = np.sum(vis.shape[:2]) * 0.001 * size
         size = max(1, size)
     size = int(round(size))
+    if hasattr(color, "tolist"):
+        color = tuple(color.tolist())
     _color = (255, 0, 0) if color is None else color
     _draw_contour = False
     if convex_hull:
@@ -429,7 +496,13 @@ def vis_point_uvs(
     for idx, uv in enumerate(uvs):
         if color is None:
             vis = cv2.circle(vis, tuple(uv[:2]), size * 2, (255, 255, 255), -1)
-        vis = cv2.circle(vis, tuple(uv[:2]), size, _color, -1,)
+        vis = cv2.circle(
+            vis,
+            tuple(uv[:2]),
+            size,
+            _color,
+            -1,
+        )
     return vis
 
 
@@ -456,7 +529,12 @@ def vis_T(T, cam=None, img=None, length=0.1):
     else:
         vis = img.copy()
     rvec, tvec = T_to_r_t(T)
+    invert_color = T[2, 3] < 0
+    if invert_color:
+        vis = 255 - vis
     cv2.drawFrameAxes(vis, cam.K, cam.D, rvec, tvec, length)
+    if invert_color:
+        vis = 255 - vis
     return vis
 
 
@@ -490,7 +568,13 @@ def vis_align(img1, img2, n_line=21, shows=True):
     img2 = _to_3x_uint8(img2)
     y, x = img1.shape[:2]
     visv = vis_stereo(img1, img2, n_line=n_line)
-    vis = np.concatenate((visv, vis_stereo(img2, img1, n_line=n_line),), 0)
+    vis = np.concatenate(
+        (
+            visv,
+            vis_stereo(img2, img1, n_line=n_line),
+        ),
+        0,
+    )
     vis = np.rot90(
         vis_stereo(
             np.rot90(vis[:y]), np.rot90(vis[y:]), n_line=int(n_line * x * 2 / y)
@@ -504,7 +588,7 @@ def vis_align(img1, img2, n_line=21, shows=True):
 
 
 def get_test_cams(feature_type="checkboard"):
-    from calibrating import Cam, ArucoFeatureLib
+    from calibrating import Cam, PredifinedArucoBoard1
 
     if feature_type == "checkboard":
         caml, camr, camd = Cam.get_test_cams()
@@ -516,22 +600,22 @@ def get_test_cams(feature_type="checkboard"):
                 "../../../calibrating_example_data/paired_stereo_and_depth_cams_aruco",
             )
         )
-        feature_lib = ArucoFeatureLib(occlusion=True)
+        board = PredifinedArucoBoard1()
         caml = Cam(
             glob(os.path.join(root, "*", "stereo_l.jpg")),
-            feature_lib,
+            board,
             name="caml",
             enable_cache=True,
         )
         camr = Cam(
             glob(os.path.join(root, "*", "stereo_r.jpg")),
-            feature_lib,
+            board,
             name="camr",
             enable_cache=True,
         )
         camd = Cam(
             glob(os.path.join(root, "*", "depth_cam_color.jpg")),
-            feature_lib,
+            board,
             name="camd",
             enable_cache=True,
         )
@@ -555,8 +639,8 @@ class CvShow:
                 cvshow.imshow(rgb, "Windows name")
                 if key == "q":
                     break
-    
-    Or a more sample but not destroyAllWindows() when meet Exception 
+
+    Or a more sample but not destroyAllWindows() when meet Exception
     >>> for key in cvshow:
             cvshow.imshow(rgb, "Windows name")
             if key == "q":
